@@ -508,6 +508,7 @@ use_default_name:
 	INIT_WORK(&rdev->propagate_cac_done_wk, cfg80211_propagate_cac_done_wk);
 	INIT_WORK(&rdev->mgmt_registrations_update_wk,
 		  cfg80211_mgmt_registrations_update_wk);
+	spin_lock_init(&rdev->mgmt_registrations_lock);
 
 #ifdef CPTCFG_CFG80211_DEFAULT_PS
 	rdev->wiphy.flags |= WIPHY_FLAG_PS_ON_BY_DEFAULT;
@@ -1263,7 +1264,6 @@ void cfg80211_init_wdev(struct wireless_dev *wdev)
 	INIT_LIST_HEAD(&wdev->event_list);
 	spin_lock_init(&wdev->event_lock);
 	INIT_LIST_HEAD(&wdev->mgmt_registrations);
-	spin_lock_init(&wdev->mgmt_registrations_lock);
 	INIT_LIST_HEAD(&wdev->pmsr_list);
 	spin_lock_init(&wdev->pmsr_lock);
 	INIT_WORK(&wdev->pmsr_free_wk, cfg80211_pmsr_free_wk);
@@ -1272,11 +1272,6 @@ void cfg80211_init_wdev(struct wireless_dev *wdev)
 #ifdef CONFIG_WIRELESS_EXT
 	if (!wdev->netdev->wireless_handlers)
 		wdev->netdev->wireless_handlers = &cfg80211_wext_handler;
-#else
-	printk_once(KERN_WARNING "cfg80211: wext will not work because "
-		    "kernel was compiled with CONFIG_WIRELESS_EXT=n. "
-		    "Tools using wext interface, like iwconfig will "
-		    "not work.\n");
 #endif
 	wdev->wext.default_key = -1;
 	wdev->wext.default_mgmt_key = -1;
